@@ -188,36 +188,11 @@
 #define CopyCursor(pcur) ((HCURSOR)CopyIcon((HICON)(pcur)))
 #endif
 
-/* Define corresponding Windows types for some X11 types, just for laziness. */
-typedef PALETTEENTRY XColor;
-
-/* Some structs are somewhat useful to emulate internally. */
-typedef struct {
-  HPALETTE palette;		/* Palette handle used when drawing. */
-  guint size;			/* Number of entries in the palette. */
-  gboolean stale;		/* 1 if palette needs to be realized,
-				 * otherwise 0. */
-  gboolean *in_use;
-  gboolean rc_palette;		/* If RC_PALETTE is on in the RASTERCAPS */
-  gulong sizepalette;		/* SIZEPALETTE if rc_palette */
-} ColormapStruct, *Colormap;
-  
-typedef struct {
-  gint map_entries;
-  guint visualid;
-  guint bitspixel;
-} Visual;
-
-typedef struct {
-  Colormap colormap;
-  unsigned long red_max;
-  unsigned long red_mult;
-  unsigned long green_max;
-  unsigned long green_mult;
-  unsigned long blue_max;
-  unsigned long blue_mult;
-  unsigned long base_pixel;
-} XStandardColormap;
+/* Define some combinations of GdkDebugFlags */
+#define GDK_DEBUG_EVENTS_OR_COLORMAP (GDK_DEBUG_EVENTS|GDK_DEBUG_COLORMAP)
+#define GDK_DEBUG_EVENTS_OR_INPUT (GDK_DEBUG_EVENTS|GDK_DEBUG_INPUT)
+#define GDK_DEBUG_PIXMAP_OR_COLORMAP (GDK_DEBUG_PIXMAP|GDK_DEBUG_COLORMAP)
+#define GDK_DEBUG_MISC_OR_COLORMAP (GDK_DEBUG_MISC|GDK_DEBUG_COLORMAP)
 
 #define GDK_TYPE_GC_WIN32              (_gdk_gc_win32_get_type ())
 #define GDK_GC_WIN32(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_GC_WIN32, GdkGCWin32))
@@ -230,10 +205,8 @@ typedef struct _GdkColormapPrivateWin32 GdkColormapPrivateWin32;
 typedef struct _GdkCursorPrivate        GdkCursorPrivate;
 typedef struct _GdkWin32SingleFont      GdkWin32SingleFont;
 typedef struct _GdkFontPrivateWin32     GdkFontPrivateWin32;
-typedef struct _GdkImagePrivateWin32    GdkImagePrivateWin32;
-typedef struct _GdkVisualPrivate        GdkVisualPrivate;
-typedef struct _GdkGCWin32      GdkGCWin32;
-typedef struct _GdkGCWin32Class GdkGCWin32Class;
+typedef struct _GdkGCWin32		GdkGCWin32;
+typedef struct _GdkGCWin32Class		GdkGCWin32Class;
 
 struct _GdkCursorPrivate
 {
@@ -261,25 +234,21 @@ struct _GdkVisualClass
   GObjectClass parent_class;
 };
 
-struct _GdkVisualPrivate
-{
-  GdkVisual visual;
-  Visual *xvisual;
-};
+typedef enum {
+  GDK_WIN32_PE_STATIC,
+  GDK_WIN32_PE_AVAILABLE,
+  GDK_WIN32_PE_INUSE
+} GdkWin32PalEntryState;
 
 struct _GdkColormapPrivateWin32
 {
-  Colormap xcolormap;
+  HPALETTE hpal;
+  gint current_size;		/* Current size of hpal */
+  GdkWin32PalEntryState *use;
   gint private_val;
 
   GHashTable *hash;
   GdkColorInfo *info;
-  DWORD last_sync_time;
-};
-
-struct _GdkImagePrivateWin32
-{
-  HBITMAP hbitmap;
 };
 
 struct _GdkGCWin32
@@ -294,9 +263,7 @@ struct _GdkGCWin32
    * window or pixmap. We thus keep all the necessary values in the
    * GdkGCWin32 object.
    */
-  HDC hdc;
 
-  GdkRegion *clip_region;
   HRGN hcliprgn;
 
   GdkGCValuesMask values_mask;
@@ -313,10 +280,17 @@ struct _GdkGCWin32
   gint graphics_exposures;
   gint pen_width;
   DWORD pen_style;
-  HANDLE hwnd;			/* If a HDC is allocated, for which window,
-				 * or what bitmap is selected into it
-				 */
+  DWORD *pen_dashes;		/* use for PS_USERSTYLE or step-by-step rendering */
+  gint pen_num_dashes;
+
+  /* Following fields are valid while the GC exists as a Windows DC */
+  HDC hdc;
   int saved_dc;
+
+  HANDLE hwnd;			/* For which window, or what bitmap is
+				 * selected into it
+				 */
+  HPALETTE holdpal;
 };
 
 struct _GdkGCWin32Class
@@ -326,60 +300,69 @@ struct _GdkGCWin32Class
 
 GType _gdk_gc_win32_get_type (void);
 
-/* Routines from gdkgeometry-win32.c */
-void
-_gdk_window_init_position (GdkWindow *window);
-void
-_gdk_window_move_resize_child (GdkWindow *window,
-			       gint       x,
-			       gint       y,
-			       gint       width,
-			       gint       height);
-void _gdk_window_process_expose    (GdkWindow     *window,
-                                    gulong         serial,
-                                    GdkRectangle  *area);
+gulong _gdk_win32_get_next_tick (gulong suggested_tick);
 
-/* gdkdrawable-win32.c, background draw helper */
-void _gdk_win32_draw_tiles (GdkDrawable *drawable,
-                            GdkGC       *gc,
-                            GdkPixmap   *tile,
-                            gint        x, 
-                            gint        y, 
-                            gint        width, 
-                            gint        height);
+void _gdk_window_init_position     (GdkWindow *window);
+void _gdk_window_move_resize_child (GdkWindow *window,
+				    gint       x,
+				    gint       y,
+				    gint       width,
+				    gint       height);
+void _gdk_window_process_expose    (GdkWindow *window,
+                                    GdkRegion *invalidate_region);
 
 void _gdk_win32_selection_init (void);
 void _gdk_win32_dnd_exit (void);
 
-void	 gdk_win32_handle_table_insert    (HANDLE   *handle,
-					   gpointer data);
-void	 gdk_win32_handle_table_remove    (HANDLE handle);
+void	 gdk_win32_handle_table_insert  (HANDLE   *handle,
+					 gpointer data);
+void	 gdk_win32_handle_table_remove  (HANDLE handle);
 
-GdkGC *  _gdk_win32_gc_new       (GdkDrawable        *drawable,
-				  GdkGCValues        *values,
-				  GdkGCValuesMask     values_mask);
+GdkGC    *_gdk_win32_gc_new             (GdkDrawable        *drawable,
+					 GdkGCValues        *values,
+					 GdkGCValuesMask     values_mask);
 
-GdkImage* _gdk_win32_get_image (GdkDrawable    *drawable,
-				gint            x,
-				gint            y,
-				gint            width,
-				gint            height);
+GdkImage *_gdk_win32_get_image 		(GdkDrawable *drawable,
+					 gint         x,
+					 gint         y,
+					 gint         width,
+					 gint         height);
 
-GdkImage *_gdk_win32_copy_to_image       (GdkDrawable *drawable,
-					  GdkImage    *image,
-					  gint         src_x,
-					  gint         src_y,
-					  gint         dest_x,
-					  gint         dest_y,
-					  gint         width,
-					  gint         height);
+GdkImage *_gdk_win32_copy_to_image      (GdkDrawable *drawable,
+					 GdkImage    *image,
+					 gint         src_x,
+					 gint         src_y,
+					 gint         dest_x,
+					 gint         dest_y,
+					 gint         width,
+					 gint         height);
 
-COLORREF _gdk_win32_colormap_color       (GdkColormap *colormap,
-				          gulong       pixel);
+GdkImage *_gdk_win32_setup_pixmap_image (GdkPixmap *pixmap,
+					 GdkWindow *window,
+					 gint       width,
+					 gint       height,
+					 gint       depth,
+					 guchar    *bits);
 
-HRGN	 BitmapToRegion                  (HBITMAP hBmp);
+void      _gdk_win32_blit               (gboolean              use_fg_bg,
+					 GdkDrawableImplWin32 *drawable,
+					 GdkGC       	       *gc,
+					 GdkDrawable   	       *src,
+					 gint        	    	xsrc,
+					 gint        	    	ysrc,
+					 gint        	    	xdest,
+					 gint        	    	ydest,
+					 gint        	    	width,
+					 gint        	    	height);
 
-gchar   *gdk_font_full_name_get          (GdkFont *font);
+COLORREF  _gdk_win32_colormap_color     (GdkColormap *colormap,
+				         gulong       pixel);
+
+HRGN	  _gdk_win32_bitmap_to_hrgn     (GdkPixmap   *bitmap);
+
+HRGN	  _gdk_win32_gdkregion_to_hrgn  (GdkRegion   *region,
+					 gint         x_origin,
+					 gint         y_origin);
 
 void    _gdk_selection_property_store (GdkWindow *owner,
                                        GdkAtom    type,
@@ -409,71 +392,116 @@ void    _gdk_wchar_text_handle    (GdkFont       *font,
 				   void          *arg);
 
 #ifdef G_ENABLE_DEBUG
-gchar *gdk_win32_color_to_string      (const        GdkColor *color);
-gchar *gdk_win32_cap_style_to_string  (GdkCapStyle  cap_style);
-gchar *gdk_win32_fill_style_to_string (GdkFill      fill);
-gchar *gdk_win32_function_to_string   (GdkFunction  function);
-gchar *gdk_win32_join_style_to_string (GdkJoinStyle join_style);
-gchar *gdk_win32_line_style_to_string (GdkLineStyle line_style);
-gchar *gdk_win32_message_name         (UINT         msg);
+gchar *_gdk_win32_color_to_string      (const GdkColor *color);
+void   _gdk_win32_print_paletteentries (const PALETTEENTRY *pep,
+					const int           nentries);
+void   _gdk_win32_print_system_palette (void);
+void   _gdk_win32_print_hpalette       (HPALETTE     hpal);
+void   _gdk_win32_print_dc             (HDC          hdc);
+
+gchar *_gdk_win32_cap_style_to_string  (GdkCapStyle  cap_style);
+gchar *_gdk_win32_fill_style_to_string (GdkFill      fill);
+gchar *_gdk_win32_function_to_string   (GdkFunction  function);
+gchar *_gdk_win32_join_style_to_string (GdkJoinStyle join_style);
+gchar *_gdk_win32_line_style_to_string (GdkLineStyle line_style);
+gchar *_gdk_win32_gcvalues_mask_to_string (GdkGCValuesMask mask);
+gchar *_gdk_win32_drawable_description (GdkDrawable *d);
+
+gchar *_gdk_win32_rop2_to_string       (int          rop2);
+gchar *_gdk_win32_lbstyle_to_string    (UINT         brush_style);
+gchar *_gdk_win32_pstype_to_string     (DWORD        pen_style);
+gchar *_gdk_win32_psstyle_to_string    (DWORD        pen_style);
+gchar *_gdk_win32_psendcap_to_string   (DWORD        pen_style);
+gchar *_gdk_win32_psjoin_to_string     (DWORD        pen_style);
+gchar *_gdk_win32_message_to_string    (UINT         msg);
+gchar *_gdk_win32_rect_to_string       (const RECT  *rect);
+
+gchar *_gdk_win32_gdkrectangle_to_string (const GdkRectangle *rect);
+gchar *_gdk_win32_gdkregion_to_string    (const GdkRegion    *box);
 
-#define PING() printf(G_STRLOC),fflush(stdout)
-#else
-#define PING()
 #endif
 
-gchar  *gdk_win32_last_error_string (void);
-void    gdk_win32_api_failed        (const gchar *where,
+gchar  *_gdk_win32_last_error_string (void);
+void    _gdk_win32_api_failed        (const gchar *where,
 				     gint line,
 				     const gchar *api);
-void    gdk_other_api_failed        (const gchar *where,
+void    _gdk_other_api_failed        (const gchar *where,
 				     gint line,
 				     const gchar *api);
-void    gdk_win32_gdi_failed        (const gchar *where,
+void    _gdk_win32_gdi_failed        (const gchar *where,
 				     gint line,
 				     const gchar *api);
 
 #ifdef __GNUC__
-#define WIN32_API_FAILED(api) gdk_win32_api_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
-#define WIN32_GDI_FAILED(api) gdk_win32_gdi_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
-#define OTHER_API_FAILED(api) gdk_other_api_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
+#define WIN32_API_FAILED(api) _gdk_win32_api_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
+#define WIN32_GDI_FAILED(api) _gdk_win32_gdi_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
+#define OTHER_API_FAILED(api) _gdk_other_api_failed (__FILE__ ":" __PRETTY_FUNCTION__, __LINE__, api)
 #else
-#define WIN32_API_FAILED(api) gdk_win32_api_failed (__FILE__, __LINE__, api)
-#define WIN32_GDI_FAILED(api) gdk_win32_gdi_failed (__FILE__, __LINE__, api)
-#define OTHER_API_FAILED(api) gdk_other_api_failed (__FILE__, __LINE__, api)
+#define WIN32_API_FAILED(api) _gdk_win32_api_failed (__FILE__, __LINE__, api)
+#define WIN32_GDI_FAILED(api) _gdk_win32_gdi_failed (__FILE__, __LINE__, api)
+#define OTHER_API_FAILED(api) _gdk_other_api_failed (__FILE__, __LINE__, api)
 #endif
  
+/* These two macros call a GDI or other Win32 API and if the return
+ * value is zero or NULL, print a warning message. The majority of GDI
+ * calls return zero or NULL on failure. The value of the macros is nonzero
+ * if the call succeeded, zero otherwise.
+ */
+
+#define GDI_CALL(api, arglist) (api arglist ? 1 : (WIN32_GDI_FAILED (#api), 0))
+#define API_CALL(api, arglist) (api arglist ? 1 : (WIN32_API_FAILED (#api), 0))
+ 
 extern LRESULT CALLBACK _gdk_win32_window_procedure (HWND, UINT, WPARAM, LPARAM);
 
-extern HWND		 gdk_root_window;
+extern HWND		 _gdk_root_window;
+extern GdkWindow        *_gdk_parent_root;
 
-extern HDC		 gdk_display_hdc;
-extern HINSTANCE	 gdk_dll_hinstance;
-extern HINSTANCE	 gdk_app_hmodule;
+extern GdkDisplay       *_gdk_display;
+extern GdkScreen        *_gdk_screen;
+
+extern HDC		 _gdk_display_hdc;
+extern HINSTANCE	 _gdk_dll_hinstance;
+extern HINSTANCE	 _gdk_app_hmodule;
+
+/* These are thread specific, but GDK/win32 works OK only when invoked
+ * from a single thread anyway.
+ */
+extern HKL		 _gdk_input_locale;
+extern UINT		 _gdk_input_codepage;
+
+extern guint		 _gdk_keymap_serial;
 
 /* Registered clipboard formats */
-extern WORD		 cf_rtf;
-extern WORD		 cf_utf8_string;
+extern WORD		 _cf_rtf;
+extern WORD		 _cf_utf8_string;
 
 /* GdkAtoms: Targets */
-extern GdkAtom           utf8_string;
-extern GdkAtom		 compound_text;
-extern GdkAtom		 text_uri_list;
+extern GdkAtom           _utf8_string;
+extern GdkAtom		 _compound_text;
+extern GdkAtom		 _text_uri_list;
 
 /* DND selections */
-extern GdkAtom           local_dnd;
-extern GdkAtom		 gdk_win32_dropfiles;
-extern GdkAtom		 gdk_ole2_dnd;
+extern GdkAtom           _local_dnd;
+extern GdkAtom		 _gdk_win32_dropfiles;
+extern GdkAtom		 _gdk_ole2_dnd;
 
 extern GdkAtom		 _gdk_selection_property;
 
-extern DWORD		 windows_version;
-#define IS_WIN_NT()      (windows_version < 0x80000000)
+extern DWORD		 _windows_version;
+#define IS_WIN_NT()      (_windows_version < 0x80000000)
 
 /* Options */
-extern gboolean		 gdk_input_ignore_wintab;
-extern gboolean		 gdk_event_func_from_window_proc;
+extern gboolean		 _gdk_input_ignore_wintab;
+extern gint		 _gdk_max_colors;
 
-#define IMAGE_PRIVATE_DATA(image) ((GdkImagePrivateWin32 *) GDK_IMAGE (image)->windowing_data)
+#define GDK_WIN32_COLORMAP_DATA(cmap) ((GdkColormapPrivateWin32 *) GDK_COLORMAP (cmap)->windowing_data)
+
+/* Initialization */
+void _gdk_windowing_window_init (void);
+void _gdk_visual_init (void);
+void _gdk_dnd_init    (void);
+void _gdk_windowing_image_init  (void);
+void _gdk_events_init (void);
+void _gdk_input_init  (GdkDisplay *display);
 
 #endif /* __GDK_PRIVATE_WIN32_H__ */
